==> support-pager-adapters-1.0.0-sources.jar: universum/studios/android/support/pager/adapter/FragmentStatePagerAdapter.java <==
/*
 * =================================================================================================
 *                       Copyright (C) 2011 The Android Open Source Project
 * =================================================================================================
 *                             Copyright (C) Universum Studios
 * =================================================================================================
 *         Licensed under the Apache License, Version 2.0 or later (further "License" only).
 * -------------------------------------------------------------------------------------------------
 * You may use this file only in compliance with the License. More details and copy of this License
 * you may obtain at
 *
 * 		http://www.apache.org/licenses/LICENSE-2.0
 *
 * You can redistribute, modify or publish any part of the code written within this file but as it
 * is described in the License, the software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES or CONDITIONS OF ANY KIND.
 *
 * See the License for the specific language governing permissions and limitations under the License.
 * =================================================================================================
 */
package universum.studios.android.support.pager.adapter;

import android.annotation.SuppressLint;
import android.os.Build;
import android.os.Bundle;
import android.os.Parcelable;
import android.support.annotation.IdRes;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentTransaction;
import android.support.v4.view.PagerAdapter;
import android.util.Log;
import android.view.View;
import android.view.ViewGroup;

import java.util.ArrayList;
import java.util.List;

/**
 * A {@link PagerAdapter} implementation that represents each page as a {@link Fragment}. This pager
 * adapter also handles saving and restoring state of its fragment.
 * <p>
 * This version of the pager adapter is more useful when there are a large number of pages, working
 * more like a list view. When pages are not visible to a user, their entire fragment may be destroyed,
 * only keeping the saved state of that fragment. This allows to the pager to hold on to much less
 * memory associated with each visited page as compared to {@link FragmentPagerAdapter} at the cost
 * of potentially more overhead when switching between pages.
 * <p>
 * <b>Note</b>, that when using FragmentPagerAdapter, the host ViewPager must have a valid ID set,
 * otherwise this adapter implementation will throw an exception.
 * <p>
 * The inheritance hierarchies only need to implement {@link #getItem(int)} and
 * {@link #getCount()} to become a fully working adapters.
 */
@SuppressLint("LongLogTag")
public abstract class FragmentStatePagerAdapter extends PagerAdapter {

	/**
	 * Interface ===================================================================================
	 */

	/**
	 * Constants ===================================================================================
	 */

	/**
	 * Log TAG.
	 */
	private static final String TAG = "FragmentStatePagerAdapter";

	/**
	 * Bundle key used to store saved states within this adapter's saved state.
	 *
	 * @see #saveState()
	 * @see #restoreState(Parcelable, ClassLoader)
	 */
	private static final String BUNDLE_SAVED_STATES = TAG + ".BUNDLE.SavedStates";

	/**
	 * Static members ==============================================================================
	 */

	/**
	 * Members =====================================================================================
	 */

	/**
	 * Fragment manager used to manage fragments provided by this pager adapter.
	 */
	private final FragmentManager mFragmentManager;

	/**
	 * Boolean flag indicating whether this pager adapter should create tags for its items or not.
	 *
	 * @see #makeItemTag(int, long)
	 */
	private boolean mMakeItemTags = false;

	/**
	 * Current pending fragment transaction to be committed in {@link #finishUpdate(ViewGroup)}.
	 */
	private FragmentTransaction mPendingTransaction = null;

	/**
	 * List used to store saved state for by this adapter instantiated fragments.
	 */
	private final List<Fragment.SavedState> mSavedState = new ArrayList<>(10);

	/**
	 * List used to store by this adapter instantiated fragments.
	 */
	private final List<Fragment> mFragments = new ArrayList<>(10);

	/**
	 * Fragment that is currently the primary item in the associated pager.
	 */
	Fragment mPrimaryItem = null;

	/**
	 * Constructors ================================================================================
	 */

	/**
	 * Creates a new instance of FragmentStatePagerAdapter with the specified <var>fragmentManager</var>.
	 *
	 * @param fragmentManager The manager used to manage fragments provided by the pager adapter.
	 */
	public FragmentStatePagerAdapter(@NonNull FragmentManager fragmentManager) {
		this.mFragmentManager = fragmentManager;
	}

	/**
	 * Methods =====================================================================================
	 */

	/**
	 */
	@Override
	public void startUpdate(ViewGroup container) {
		if (container.getId() == View.NO_ID) throw new IllegalStateException(
				"ViewPager with adapter " + this + " requires a view id!"
		);
	}

	/**
	 */
	@Override
	@SuppressLint("CommitTransaction")
	public Object instantiateItem(ViewGroup container, int position) {
		// If we already have this item instantiated, there is nothing to do. This can happen when
		// we are restoring the entire pager from its saved state, where the fragment manager has
		// already taken care of restoring the fragments we previously had instantiated.
		if (mFragments.size() > position) {
			final Fragment fragment = mFragments.get(position);
			if (fragment != null) return fragment;
		}
		if (mPendingTransaction == null) {
			// Will be committed in call to finishUpdate(...).
			this.mPendingTransaction = mFragmentManager.beginTransaction();
		}
		final long itemId = getItemId(position);
		final Fragment fragment = getItem(position);
		if (mSavedState.size() > position) {
			final Fragment.SavedState fragmentSavedState = mSavedState.get(position);
			if (fragmentSavedState != null) {
				fragment.setInitialSavedState(fragmentSavedState);
			}
		}
		while (mFragments.size() <= position) {
			mFragments.add(null);
		}
		fragment.setMenuVisibility(false);
		FragmentPagerAdapter.setUserVisibleHint(fragment, false);
		mFragments.set(position, fragment);
		if (PagerAdaptersConfig.DEBUG_LOG_ENABLED) {
			Log.v(TAG, "Adding item(id: " + itemId + ", fragment: " + fragment + ").");
		}
		final int containerId = container.getId();
		mPendingTransaction.add(containerId, fragment, makeItemTag(containerId, itemId));
		return fragment;
	}

	/**
	 * Sets a boolean flag indicating whether this pager adapter should make tags for its items or
	 * not.
	 * <p>
	 * This feature is by default <b>disabled</b>.
	 *
	 * @param makeItemTags {@code True} to enable making of item tags, {@code false} otherwise.
	 * @see #makeItemTag(int, long)
	 */
	protected final void setMakeItemTags(boolean makeItemTags) {
		this.mMakeItemTags = makeItemTags;
	}

	/**
	 * Makes a tag for an item with the given <var>itemId</var>.
	 * <p>
	 * By default making of item tags is <b>disabled</b> and the default implementation makes tag
	 * in the following format:
	 * <pre>
	 * "android:pager:CONTAINER_ID:ITEM_ID"
	 * </pre>
	 * This feature may be enabled/disabled via {@link #setMakeItemTags(boolean)}.
	 *
	 * @param containerId Id of the associated view pager container.
	 * @param itemId      Id of the item for which to create its corresponding tag.
	 * @return Item's tag or {@code null} if this adapter does not create tags for its items.
	 */
	@Nullable
	protected String makeItemTag(@IdRes int containerId, long itemId) {
		return mMakeItemTags ? "android:pager:" + containerId + ":" + itemId : null;
	}

	/**
	 * Returns a unique identifier for the item at the specified <var>position</var>.
	 * <p>
	 * The default implementation returns the given position. The inheritance hierarchies should
	 * override this method if the positions of items can change.
	 *
	 * @param position Position from the range of size of this this adapter.
	 * @return Unique identifier for the item at the requested position.
	 */
	public long getItemId(int position) {
		return position;
	}

	/**
	 * Instantiates a new {@link Fragment} associated with the specified <var>position</var>.
	 *
	 * @see #instantiateItem(ViewGroup, int)
	 */
	@NonNull
	public abstract Fragment getItem(int position);

	/**
	 */
	@Override
	public void setPrimaryItem(ViewGroup container, int position, Object object) {
		final Fragment fragment = (Fragment) object;
		if (mPrimaryItem != fragment) {
			if (mPrimaryItem != null) {
				this.mPrimaryItem.setMenuVisibility(false);
				FragmentPagerAdapter.setUserVisibleHint(mPrimaryItem, false);
			}
			if (fragment != null) {
				fragment.setMenuVisibility(true);
				FragmentPagerAdapter.setUserVisibleHint(fragment, true);
			}
			this.mPrimaryItem = fragment;
		}
	}

	/**
	 */
	@Override
	public boolean isViewFromObject(View view, Object object) {
		return ((Fragment) object).getView() == view;
	}

	/**
	 */
	@Override
	@SuppressLint("CommitTransaction")
	public void destroyItem(ViewGroup container, int position, Object object) {
		if (mPendingTransaction == null) {
			// Will be committed in call to finishUpdate(...).
			this.mPendingTransaction = mFragmentManager.beginTransaction();
		}
		final Fragment fragment = (Fragment) object;
		if (PagerAdaptersConfig.DEBUG_LOG_ENABLED) {
			Log.v(TAG, "Removing item(id: " + getItemId(position) + ", fragment: " + fragment + ", view: " + fragment.getView() + ").");
		}
		while (mSavedState.size() <= position) {
			this.mSavedState.add(null);
		}
		this.mSavedState.set(
				position,
				fragment.isAdded() ? mFragmentManager.saveFragmentInstanceState(fragment) : null
		);
		this.mFragments.set(position, null);
		this.mPendingTransaction.remove(fragment);
	}

	/**
	 */
	@Override
	public void finishUpdate(ViewGroup container) {
		if (mPendingTransaction != null) {
			if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
				this.mPendingTransaction.commitNowAllowingStateLoss();
			} else {
				this.mPendingTransaction.commitAllowingStateLoss();
			}
			this.mPendingTransaction = null;
		}
	}

	/**
	 */
	@Override
	public Parcelable saveState() {
		Bundle state = null;
		if (mSavedState.size() > 0) {
			state = new Bundle();
			final Fragment.SavedState[] savedStates = new Fragment.SavedState[mSavedState.size()];
			mSavedState.toArray(savedStates);
			state.putParcelableArray(BUNDLE_SAVED_STATES, savedStates);
		}
		for (int i = 0; i < mFragments.size(); i++) {
			final Fragment fragment = mFragments.get(i);
			if (fragment != null && fragment.isAdded()) {
				if (state == null) {
					state = new Bundle();
				}
				mFragmentManager.putFragment(state, "f" + i, fragment);
			}
		}
		return state;
	}

	/**
	 */
	@Override
	public void restoreState(Parcelable state, ClassLoader loader) {
		if (state != null) {
			Bundle bundle = (Bundle) state;
			bundle.setClassLoader(loader);
			final Parcelable[] savedStates = bundle.getParcelableArray(BUNDLE_SAVED_STATES);
			mSavedState.clear();
			mFragments.clear();
			if (savedStates != null && savedStates.length > 0) {
				for (final Parcelable savedState : savedStates) {
					mSavedState.add((Fragment.SavedState) savedState);
				}
			}
			final Iterable<String> keys = bundle.keySet();
			for (String key : keys) {
				if (key.startsWith("f")) {
					int index = Integer.parseInt(key.substring(1));
					final Fragment fragment = mFragmentManager.getFragment(bundle, key);
					if (fragment != null) {
						while (mFragments.size() <= index) {
							mFragments.add(null);
						}
						fragment.setMenuVisibility(false);
						mFragments.set(index, fragment);
					} else {
						Log.w(TAG, "Bad fragment at key " + key);
					}
				}
			}
		}
	}

	/**
	 * Inner classes ===============================================================================
	 */
}

==> support-pager-adapters-2.0.1-sources.jar: universum/studios/android/support/pager/adapter/FragmentStatePagerAdapter.java <==
/*
 * =================================================================================================
 *                       Copyright (C) 2011 The Android Open Source Project
 * =================================================================================================
 *                             Copyright (C) Universum Studios
 * =================================================================================================
 *         Licensed under the Apache License, Version 2.0 or later (further "License" only).
 * -------------------------------------------------------------------------------------------------
 * You may use this file only in compliance with the License. More details and copy of this License
 * you may obtain at
 *
 * 		http://www.apache.org/licenses/LICENSE-2.0
 *
 * You can redistribute, modify or publish any part of the code written within this file but as it
 * is described in the License, the software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES or CONDITIONS OF ANY KIND.
 *
 * See the License for the specific language governing permissions and limitations under the License.
 * =================================================================================================
 */
package universum.studios.android.support.pager.adapter;

import android.annotation.SuppressLint;
import android.database.Cursor;
import android.os.Bundle;
import android.os.Parcelable;
import android.support.annotation.IdRes;
import android.support.annotation.IntRange;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentTransaction;
import android.support.v4.util.LongSparseArray;
import android.support.v4.view.PagerAdapter;
import android.util.Log;
import android.view.View;
import android.view.ViewGroup;

/**
 * A {@link PagerAdapter} implementation that represents each page as a {@link Fragment}. This pager
 * adapter also handles saving and restoring state of its fragments.
 * <p>
 * This version of the pager adapter is more useful when there are a large number of pages, working
 * more like a list view. When pages are not visible to a user, their entire fragment may be destroyed,
 * only keeping the saved state of that fragment. This allows to the pager to hold on to much less
 * memory associated with each visited page as compared to {@link FragmentPagerAdapter} at the cost
 * of potentially more overhead when switching between pages.
 * <p>
 * <b>Note</b>, that when using FragmentStatePagerAdapter, the host ViewPager must have a valid ID
 * set, otherwise this adapter implementation will throw an exception.
 * <p>
 * The inheritance hierarchies only need to implement {@link #getItem(int)} and {@link #getCount()}
 * to become a fully working adapters.
 *
 * <h3>Not persistent data set</h3>
 * If this adapter is used with <b>not-persistent</b> data set like {@link Cursor} it is essential
 * to override also {@link #getItemId(int)} and provide item id for the requested position as this
 * adapter tracks and stores its created fragments by theirs associated item ids. Item ids are also
 * used to store states of the adapter's fragments. Also note, that {@link #getItemPosition(Object)}
 * method should be implemented in a way which can properly resolve whether a fragment should be
 * still visible at its current position, moved to a different position or to be completely removed
 * due to changes in the data set.
 */
@SuppressLint("LongLogTag")
public abstract class FragmentStatePagerAdapter extends PagerAdapter {

	/**
	 * Constants ===================================================================================
	 */

	/**
	 * Log TAG.
	 */
	private static final String TAG = "FragmentStatePagerAdapter";

	/**
	 * Bundle key used to store states of fragment instances created by this adapter.
	 *
	 * @see #saveState()
	 * @see #restoreState(Parcelable, ClassLoader)
	 */
	private static final String STATE_FRAGMENT_STATES = FragmentStatePagerAdapter.class.getName() + ".STATE.FragmentStates";

	/**
	 * Interface ===================================================================================
	 */

	/**
	 * Static members ==============================================================================
	 */

	/**
	 * Members =====================================================================================
	 */

	/**
	 * Fragment manager used to manage fragments provided by this pager adapter.
	 */
	private final FragmentManager mFragmentManager;

	/**
	 * Boolean flag indicating whether this pager adapter should create tags for its items or not.
	 *
	 * @see #makeItemTag(int, long)
	 */
	private boolean mMakeItemTags = true;

	/**
	 * Current pending fragment transaction to be committed in {@link #finishUpdate(ViewGroup)}.
	 */
	private FragmentTransaction mPendingTransaction;

	/**
	 * List used to store by this adapter instantiated fragments.
	 */
	private final LongSparseArray<Fragment> mFragments = new LongSparseArray<>(5);

	/**
	 * List used to store saved state for by this adapter instantiated fragments.
	 */
	private final LongSparseArray<Fragment.SavedState> mSavedStates = new LongSparseArray<>(5);

	/**
	 * Fragment that is currently the primary item in the associated pager.
	 */
	private Fragment mPrimaryItem;

	/**
	 * Position of the current primary item.
	 */
	private int mPrimaryPosition = -1;

	/**
	 * Constructors ================================================================================
	 */

	/**
	 * Creates a new instance of FragmentStatePagerAdapter with the specified <var>fragmentManager</var>.
	 *
	 * @param fragmentManager The manager used to manage fragments provided by the pager adapter.
	 */
	public FragmentStatePagerAdapter(@NonNull FragmentManager fragmentManager) {
		super();
		this.mFragmentManager = fragmentManager;
	}

	/**
	 * Methods =====================================================================================
	 */

	/**
	 */
	@Override
	public void startUpdate(ViewGroup container) {
		if (container.getId() == View.NO_ID) throw new IllegalStateException(
				"ViewPager with adapter " + this + " requires a view id!"
		);
	}

	/**
	 */
	@Override
	@SuppressLint("CommitTransaction")
	public Object instantiateItem(ViewGroup container, int position) {
		final long itemId = getItemId(position);
		Fragment fragment = mFragments.get(itemId);
		// If we already have this item instantiated, there is nothing to do. This can happen when
		// we are restoring the entire pager from its saved state, where the fragment manager has
		// already taken care of restoring the fragments we previously had instantiated.
		if (fragment != null) {
			return fragment;
		}
		if (mPendingTransaction == null) {
			// Will be committed in call to finishUpdate(...).
			this.mPendingTransaction = mFragmentManager.beginTransaction();
		}
		fragment = getItem(position);
		final Fragment.SavedState savedState = mSavedStates.get(itemId);
		if (savedState != null) {
			fragment.setInitialSavedState(savedState);
		}
		fragment.setMenuVisibility(false);
		PagerAdapterHelper.setUserVisibleHint(fragment, false);
		mFragments.put(itemId, fragment);
		if (PagerAdaptersConfig.DEBUG_LOG_ENABLED) {
			Log.v(TAG, "Adding item(id: " + itemId + ", fragment: " + fragment + ").");
		}
		final int containerId = container.getId();
		mPendingTransaction.add(containerId, fragment, makeItemTag(containerId, itemId));
		return fragment;
	}

	/**
	 * Sets a boolean flag indicating whether this pager adapter should make tags for its items or
	 * not.
	 * <p>
	 * This feature is by default <b>enabled</b>.
	 *
	 * @param makeItemTags {@code True} to enable making of item tags, {@code false} otherwise.
	 * @see #makeItemTag(int, long)
	 */
	protected final void setMakeItemTags(boolean makeItemTags) {
		this.mMakeItemTags = makeItemTags;
	}

	/**
	 * Makes a tag for an item with the given <var>itemId</var>.
	 * <p>
	 * By default making of item tags is <b>disabled</b> and the default implementation makes tag
	 * in the following format:
	 * <pre>
	 * "android:pager:CONTAINER_ID:ITEM_ID"
	 * </pre>
	 * This feature may be enabled/disabled via {@link #setMakeItemTags(boolean)}.
	 *
	 * @param containerId Id of the associated view pager container.
	 * @param itemId      Id of the item for which to create its corresponding tag.
	 * @return Item's tag or {@code null} if this adapter does not create tags for its items.
	 */
	@Nullable
	protected String makeItemTag(@IdRes int containerId, long itemId) {
		return mMakeItemTags ? "android:pager:" + containerId + ":" + itemId : null;
	}

	/**
	 * Returns a unique identifier for the item at the specified <var>position</var>.
	 * <p>
	 * The default implementation returns the given position. The inheritance hierarchies should
	 * override this method if the positions of items can change.
	 *
	 * @param position Position from the range of size of this this adapter.
	 * @return Unique identifier for the item at the requested position.
	 */
	public long getItemId(int position) {
		return position;
	}

	/**
	 * Instantiates a new {@link Fragment} associated with the specified <var>position</var>.
	 *
	 * @see #instantiateItem(ViewGroup, int)
	 */
	@NonNull
	public abstract Fragment getItem(int position);

	/**
	 */
	@Override
	public void setPrimaryItem(ViewGroup container, int position, Object object) {
		final Fragment fragment = (Fragment) object;
		if (mPrimaryItem != fragment) {
			this.mPrimaryPosition = position;
			if (mPrimaryItem != null) {
				this.mPrimaryItem.setMenuVisibility(false);
				PagerAdapterHelper.setUserVisibleHint(mPrimaryItem, false);
			}
			if (fragment != null) {
				fragment.setMenuVisibility(true);
				PagerAdapterHelper.setUserVisibleHint(fragment, true);
			}
			this.mPrimaryItem = fragment;
		}
	}

	/**
	 * Returns the fragment instance that has been set as primary item via {@link #setPrimaryItem(ViewGroup, int, Object)}.
	 *
	 * @return Currently primary fragment. May be {@code null} if no primary item has been set yet.
	 * @see #getPrimaryPosition()
	 */
	@Nullable
	public Fragment getPrimaryFragment() {
		return mPrimaryItem;
	}

	/**
	 * Returns position of the current primary item.
	 *
	 * @return Primary item's position or {@code -1} if no primary item has been specified yet.
	 * @see #getPrimaryFragment()
	 * @see #setPrimaryItem(ViewGroup, int, Object)
	 */
	@IntRange(from = -1)
	public int getPrimaryPosition() {
		return mPrimaryPosition;
	}

	/**
	 */
	@Override
	public boolean isViewFromObject(View view, Object object) {
		return ((Fragment) object).getView() == view;
	}

	/**
	 */
	@Override
	@SuppressLint("CommitTransaction")
	public void destroyItem(ViewGroup container, int position, Object object) {
		if (mPendingTransaction == null) {
			// Will be committed in call to finishUpdate(...).
			this.mPendingTransaction = mFragmentManager.beginTransaction();
		}
		final Fragment fragment = (Fragment) object;
		final int itemIndex = mFragments.indexOfValue(fragment);
		if (itemIndex != -1) {
			final long itemId = mFragments.keyAt(itemIndex);
			if (PagerAdaptersConfig.DEBUG_LOG_ENABLED) {
				Log.v(TAG, "Removing item(id: " + itemId + ", fragment: " + fragment + ", view: " + fragment.getView() + ").");
			}
			if (fragment.isAdded()) {
				this.mSavedStates.put(itemId, mFragmentManager.saveFragmentInstanceState(fragment));
			} else {
				this.mSavedStates.remove(itemId);
			}
			this.mFragments.remove(itemId);
		}
		this.mPendingTransaction.remove(fragment);
	}

	/**
	 */
	@Override
	public void finishUpdate(ViewGroup container) {
		if (mPendingTransaction != null) {
			PagerAdapterHelper.finishUpdate(mPendingTransaction);
			this.mPendingTransaction = null;
		}
	}

	/**
	 */
	@Override
	public Parcelable saveState() {
		Bundle state = null;
		final int statesCount = mSavedStates.size();
		if (statesCount > 0) {
			state = new Bundle();
			final long[] savedStateIds = new long[statesCount];
			for (int i = 0; i <statesCount; i++) {
				final Fragment.SavedState savedState = mSavedStates.valueAt(i);
				savedStateIds[i] = mSavedStates.keyAt(i);
				state.putParcelable("android:pager:fragment:state:" + savedStateIds[i], savedState);
			}
			state.putLongArray(STATE_FRAGMENT_STATES, savedStateIds);
		}
		for (int i = 0; i < mFragments.size(); i++) {
			final Fragment fragment = mFragments.get(i);
			if (fragment != null && fragment.isAdded()) {
				if (state == null) {
					state = new Bundle();
				}
				mFragmentManager.putFragment(state, "android:pager:fragment:" + mFragments.keyAt(i), fragment);
			}
		}
		return state;
	}

	/**
	 */
	@Override
	public void restoreState(Parcelable state, ClassLoader loader) {
		if (state != null) {
			final Bundle bundle = (Bundle) state;
			bundle.setClassLoader(loader);
			final long[] savedStateIds = bundle.getLongArray(STATE_FRAGMENT_STATES);
			mSavedStates.clear();
			mFragments.clear();
			if (savedStateIds != null && savedStateIds.length > 0) {
				for (final long savedStateId : savedStateIds) {
					mSavedStates.put(savedStateId, bundle.<Fragment.SavedState>getParcelable("android:pager:fragment:state:" + savedStateId));
				}
			}
			final Iterable<String> keys = bundle.keySet();
			for (final String key : keys) {
				if (key.startsWith("android:pager:fragment:")) {
					final Fragment fragment = mFragmentManager.getFragment(bundle, key);
					if (fragment != null) {
						fragment.setMenuVisibility(false);
						mFragments.put(Long.parseLong(key.split(":")[3]), fragment);
					} else if (PagerAdaptersConfig.LOG_ENABLED) {
						Log.w(TAG, "Fragment not found for key " + key + " when restoring adapter's state.");
					}
				}
			}
		}
	}

	/**
	 * Inner classes ===============================================================================
	 */
}
